package
Version:
v0.0.0-...-21e7071
Opens a new window with list of versions in this module.
Published: Oct 24, 2023
License: MIT
Opens a new window with license information.
Imports: 3
Opens a new window with list of imports.
Imported by: 0
Opens a new window with list of known importers.
Documentation
¶
type Ordinal interface {
Type
Ord()
}
Ordinal is the generic parent of ordinal types
Set models a set data type
Type represents some data type
Source Files
¶
Directories
¶
Click to show internal directories.
Click to hide internal directories.